(*
The main module of the Window Contents Explorer utility by Alexei Hmelnov.
----------------------------------------------------------------------------
E-Mail: alex@monster.icc.ru
http://monster.icc.ru/~alex/
----------------------------------------------------------------------------

See the file "readme.txt" for more details.

------------------------------------------------------------------------
                             IMPORTANT NOTE:
This software is provided 'as-is', without any expressed or implied warranty.
In no event will the author be held liable for any damages arising from the
use of this software.
Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:
1. The origin of this software must not be misrepresented, you must not
   claim that you wrote the original software.
2. Altered source versions must be plainly marked as such, and must not
   be misrepresented as being the original software.
3. This notice may not be removed or altered from any source
   distribution.
*)

unit WIMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,WinDescr, Buttons, ComCtrls, ToolWin;

type
  TWIMainForm = class(TForm)
    WIEdit: TMemo;
    SaveDlg: TSaveDialog;
    ToolBar: TToolBar;
    edPos: TEdit;
    ToolButton1: TToolButton;
    tbLock: TToolButton;
    ilTools: TImageList;
    ilToolsDisabled: TImageList;
    ToolButton2: TToolButton;
    tbSave: TToolButton;
    ToolButton3: TToolButton;
    tbGetWndImage: TToolButton;
    ToolButton4: TToolButton;
    tbBMP: TToolButton;
    tbWMF: TToolButton;
    ToolButton5: TToolButton;
    tbChildren: TToolButton;
    ToolButton6: TToolButton;
    tbStyleNames: TToolButton;
    ToolButton7: TToolButton;
    tbWrap: TToolButton;
    ToolButton8: TToolButton;
    tbHelp: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure sbSaveClick(Sender: TObject);
    procedure sbGetWndImageClick(Sender: TObject);
    procedure btnReadmeClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tbLockClick(Sender: TObject);
    procedure tbChildrenClick(Sender: TObject);
    procedure tbStyleNamesClick(Sender: TObject);
    procedure tbWrapClick(Sender: TObject);
    procedure ToolBarResize(Sender: TObject);
  private
    FDescrWnd: HWND;
    procedure SetDescrWnd(Wnd: HWND);
    property hDescrWnd: HWND read FDescrWnd write SetDescrWnd;
    procedure UpdateDescr;
  public
    procedure AppDeactivate(Sender: TObject);
  end;

var
  WIMainForm: TWIMainForm;

implementation

uses WIPaint,CapFm,StlNames;

{$R *.DFM}

function GetWordBreak(CP: PChar; ichCurrent,cch,code: integer): integer; stdcall;
const
  SepCh: set of Char = [' ',',','.','-','|',';',',',#9,#13,#10];
var
  C,EP: PChar;
begin
  case code of
    WB_LEFT: begin
      C := CP+ichCurrent;
      while (C>CP)and(C^ in SepCh) do
        Dec(C);
      while (C>CP) do begin
        Dec(C);
        if (C^ in SepCh) then begin
          Inc(C);
          Break;
        end ;
      end ;
      Result := C-CP;
    end ;
    WB_RIGHT: begin
      C := CP+ichCurrent;
      EP := CP+cch;
      while (C<EP)and(C^ in SepCh) do
        Inc(C);
      while (C>EP) do begin
        Inc(C);
        if (C^ in SepCh) then begin
          Dec(C);
          Break;
        end ;
      end ;
      Result := C-CP;
    end ;
    WB_ISDELIMITER:
      LongBool(Result) := ((CP+ichCurrent)^ in SepCh);
  else
    Result := ichCurrent;
  end ;
end ;

procedure TWIMainForm.SetDescrWnd(Wnd: HWND);
var
  Res: String;
begin
  if Wnd=FDescrWnd then
    Exit;
  FDescrWnd := Wnd;
  if Wnd=0 then
    Res := '?'
  else begin
    try
      StyleNamesOn := tbStyleNames.Down;
      Res := DescrWindows(Wnd,Point(0,0),0);
    except
      on E: Exception do
        Res := E.Message;
    end ;
  end ;
  WIEdit.Text := Res;
  tbGetWndImage.Enabled := Wnd<>0;
end ;

procedure TWIMainForm.UpdateDescr;
var
  W: HWND;
  lTop: integer;
begin
  W := FDescrWnd;
  FDescrWnd := 0;
  LockWindowUpdate(WIEdit.Handle);
  try
    lTop := WIEdit.Perform(EM_GETFIRSTVISIBLELINE,0,0);
    hDescrWnd := W;
    WIEdit.Perform(EM_LINESCROLL,0,lTop);
  finally
    LockWindowUpdate(0);
  end ;
end ;

procedure TWIMainForm.FormCreate(Sender: TObject);
begin
  Application.OnDeactivate := AppDeactivate;
//  btnReadmeClick(Self);
  InitWinClassDescrs;
end;

procedure TWIMainForm.AppDeactivate(Sender: TObject);
var
  P: TPoint;
begin
  if tbLock.Down then
    Exit;
  if not Visible then
    Exit;
  if not GetCursorPos(P) then
    Exit;
  edPos.Text := Format('%dx%d',[P.x,P.y]);
  hDescrWnd := WindowFromPoint(P);
end;

procedure TWIMainForm.sbSaveClick(Sender: TObject);
begin
  if not SaveDlg.Execute then
    Exit;
  WIEdit.Lines.SaveToFile(SaveDlg.FileName);
end;

procedure TWIMainForm.sbGetWndImageClick(Sender: TObject);
var
  Gr: TGraphic;
  BMP: TBitmap;
  WMF: TMetafile;
  DC: HDC;
  R,RWMF,RInv: TRect;
  IsWMF: boolean;
  W, H: integer;
  PX,PY,l: integer;
  Txt:array[0..$400] of Char;
begin
  if FDescrWnd=0 then
    Exit;
  if not GetWindowRect(FDescrWnd,R) then
    Exit;
  W := R.Right-R.Left;
  H := R.Bottom-R.Top;
  IsWMF := tbWMF.Down;
  if IsWMF then begin
    WMF := TMetafile.Create;
    Gr := WMF;
   end
  else begin
    BMP := TBitmap.Create;
    Gr := BMP;
  end ;
  try
    Gr.Width := W;
    Gr.Height := H;
    if IsWMF then begin
      l := StrECopy(Txt,'Window Information Explorer by AX')-PChar(@Txt);
      Txt[l] := #0;
      Inc(l);
      Inc(l,GetClassName(FDescrWnd,@Txt[l],SizeOf(Txt)-l-1));
      Txt[l] := ':';
      Inc(l);
      Inc(l,GetWindowText(FDescrWnd,@Txt[l],SizeOf(Txt)-l-1));
      Txt[l] := #0;
      Inc(l);
      Txt[l] := #0;
      Inc(l);
      PX := GetDeviceCaps(Canvas.Handle,LOGPIXELSX);
      PY := GetDeviceCaps(Canvas.Handle,LOGPIXELSY);
      RWMF := Bounds(0,0,MulDiv(W,PX*70{100},254), MulDiv(H,PY*70,254));
       {70, instead of 100 was chosen by the try and test method}
      DC := CreateEnhMetaFile(Canvas.Handle,Nil,@RWMF,Txt);
      SetWindowExtEx(DC,W,H,Nil);
     end
    else
      DC := BMP.Canvas.Handle;
    //SelectObject(DC,CreateRectRgn(0,0,W,H));
    ValidateRect(FDescrWnd,Nil);
    SelectClipRgn(DC,0);
    PaintWinToDC(FDescrWnd,DC,0,0,tbChildren.Down);
    InvalidateRect(FDescrWnd,Nil,true);
    if IsWMF then begin
      WMF.Handle := CloseEnhMetaFile(DC);
      WMF.Width := R.Right-R.Left;
      WMF.Height := R.Bottom-R.Top;
    end ;
    CaptureForm.WinImg.Picture.Assign(Gr);
    CaptureForm.Show;
  finally
    Gr.Free;
  end ;
end;

procedure TWIMainForm.btnReadmeClick(Sender: TObject);
begin
  hDescrWnd := 0;
  WIEdit.Lines.LoadFromFile(ExtractFilePath(ParamStr(0))+'readme.txt');
end;

procedure TWIMainForm.FormDestroy(Sender: TObject);
begin
  DoneWinClassDescrs;
end;

procedure TWIMainForm.tbLockClick(Sender: TObject);
begin
  tbLock.ImageIndex := Ord(tbLock.Down); 
end;

procedure TWIMainForm.tbChildrenClick(Sender: TObject);
begin
  tbChildren.ImageIndex := 7-Ord(tbChildren.Down);
end;

procedure TWIMainForm.tbStyleNamesClick(Sender: TObject);
begin
  tbStyleNames.ImageIndex := 8+Ord(not tbStyleNames.Down);
  UpdateDescr;
end;

procedure TWIMainForm.tbWrapClick(Sender: TObject);
const
  SBWrap: array[boolean] of TScrollStyle = (ssBoth,ssVertical);
var
  P: Pointer;
  LockOn,WrapOn: boolean;
begin
  tbWrap.ImageIndex := 10+Ord(tbWrap.Down);
  LockOn := false{LockWindowUpdate(WIEdit.Handle)};
  try
    WrapOn := tbWrap.Down;
    WIEdit.ScrollBars := SBWrap[WrapOn];
    WIEdit.WordWrap := WrapOn;
    if WrapOn then begin
      integer(P) := WIEdit.Perform(EM_GETWORDBREAKPROC,0,0);
      integer(P) := WIEdit.Perform(EM_SETWORDBREAKPROC,0,LongInt(@GetWordBreak));
      integer(P) := WIEdit.Perform(EM_GETWORDBREAKPROC,0,0);
      WIEdit.Perform(EM_FMTLINES,0,0);
    end ;
  finally
    if LockOn then
      LockWindowUpdate(0);
  end ;
end;

procedure TWIMainForm.ToolBarResize(Sender: TObject);
begin
  with ToolBar do begin
    Height := tbHelp.Top+tbHelp.Height+2*BorderWidth+5{(ButtonHeight+2*BorderWidth+3)*RowCount};
  end ;
end;

end.
