(*
The window image painting functions of the Window Contents Explorer utility
by Alexei Hmelnov.
----------------------------------------------------------------------------
E-Mail: alex@monster.icc.ru
http://monster.icc.ru/~alex/
----------------------------------------------------------------------------

See the file "readme.txt" for more details.

------------------------------------------------------------------------
                             IMPORTANT NOTE:
This software is provided 'as-is', without any expressed or implied warranty.
In no event will the author be held liable for any damages arising from the
use of this software.
Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:
1. The origin of this software must not be misrepresented, you must not
   claim that you wrote the original software.
2. Altered source versions must be plainly marked as such, and must not
   be misrepresented as being the original software.
3. This notice may not be removed or altered from any source
   distribution.
*)
unit WIPaint;

interface

uses
  Windows, Messages, Controls;

procedure PaintWinToDC(Handle: HWND; DC: HDC; X, Y: Integer; WithChildren: boolean);

implementation

var
  X0,Y0: integer;

procedure PaintWinToDC0(Handle: HWND; DC: HDC; X, Y: Integer; WithChildren: boolean);
var
  I, EdgeFlags, BorderFlags, SaveIndex: Integer;
  Width, Height: integer;
  R,CR,FR: TRect;
  HSub: HWND;
  Stl: LongInt;
begin
  Stl := GetWindowLong(Handle,GWL_STYLE);
  if Stl and WS_VISIBLE=0 then
    Exit;
  if not GetWindowRect(Handle,R) then
    Exit;
  if not GetClientRect(Handle,CR) then
    Exit;
  FR := R;
  MapWindowPoints(0, Handle, FR, 2);

  Width := CR.Right-CR.Left;
  Height := CR.Bottom-CR.Top;
  SaveIndex := SaveDC(DC);
  MoveWindowOrg(DC, X-FR.Left, Y-FR.Top);
//  ExcludeClipRect(DC, 0, 0, Width, Height);
  IntersectClipRect(DC, 0, 0, Width, Height);
  BorderFlags := 0;
  EdgeFlags := 0;
  if GetWindowLong(Handle, GWL_EXSTYLE) and WS_EX_CLIENTEDGE <> 0 then
  begin
    EdgeFlags := EDGE_SUNKEN;
    BorderFlags := BF_RECT or BF_ADJUST
  end else
  if GetWindowLong(Handle, GWL_STYLE) and WS_BORDER <> 0 then
  begin
    EdgeFlags := BDR_OUTER;
    BorderFlags := BF_RECT or BF_ADJUST or BF_MONO;
  end;
  if BorderFlags <> 0 then
  begin
    SetRect(R, 0, 0, Width, Height);
    DrawEdge(DC, R, EdgeFlags, BorderFlags);
    MoveWindowOrg(DC, R.Left, R.Top);
    IntersectClipRect(DC, 0, 0, Width, Height);
  end;
  SendMessage(Handle,WM_ERASEBKGND, DC, 0);
  SendMessage(Handle,WM_PAINT, DC, 0);
  {SendMessage(Handle,WM_NCPAINT, DC, 0);}
  RestoreDC(DC, SaveIndex);
  if WithChildren then begin
    HSub := GetWindow(Handle,GW_CHILD);
    if HSub=0 then
      Exit;
    HSub := GetWindow(HSub,GW_HWNDLAST);
    while HSub<>0 do begin
      if GetWindowRect(HSub,R) then
        PaintWinToDC0(HSub,DC,R.Left-X0,R.Top-Y0,true);
      HSub := GetWindow(HSub,GW_HWNDPREV{GW_HWNDNEXT});
    end ;
  end ;
end;

procedure PaintWinToDC(Handle: HWND; DC: HDC; X, Y: Integer; WithChildren: boolean);
var
  R: TRect;
begin
  if not GetWindowRect(Handle,R) then
    Exit;
  X0 := R.Left;
  Y0 := R.Top;
  PaintWinToDC0(Handle,DC,X,Y,WithChildren);
end;

end.
